package card

import (
	"errors"
	"fmt"
	"spaceofcheng/services/models"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/utils"
	"strconv"
	"strings"
	"time"

	"github.com/go-xorm/xorm"
	"github.com/yl10/kit/guid"
)

// CardDAO 当前数据库操作对象
type CardDAO struct {
	ctx *utils.Context
	db  *xorm.Session
}

// NewCardDAO New Inst
func NewCardDAO(ctx *utils.Context) *CardDAO {
	return &CardDAO{
		ctx: ctx,
		db:  ctx.DB,
	}
}

// CardInfo 卡
type CardInfo struct {
	model.TaCouponCard `xorm:"extends"`
	Images             []model.TaExperienceCardImage
	Targets            []model.TaCouponCardTarget
	Share              *model.TaExperienceCardShare
}

// GetCardList 获取卡列表
func (m *CardDAO) GetCardList(filters []string, limit []int) ([]model.TaCouponCard, int64, error) {
	var cds []model.TaCouponCard

	filterString := ""
	if len(filters) > 0 {
		filterString = strings.Join(filters, " and ")
		filterString += " and "
	}

	sql := `select * from ta_coupon_card where %s status > ` + strconv.Itoa(models.STATUS_DEL) + ` order by create_date desc`

	total, err := utils.NewPageNaviEngine(m.ctx).GetPageList(&cds, fmt.Sprintf(sql, filterString), limit)
	if err != nil {
		return nil, 0, err
	}

	return cds, total, nil
}

// GetCardByID 获取卡明细
func (m *CardDAO) GetCardByID(cardid string) (*CardInfo, error) {
	var card []CardInfo
	sql := `select * from ta_coupon_card where card_id=? and status>` + strconv.Itoa(models.STATUS_DEL)

	err := m.db.Sql(sql, cardid).Find(&card)
	if err != nil {
		return nil, err
	}
	if len(card) == 0 {
		return nil, nil
	}
	imgs, err := m.GetCardImgsByCardID(cardid)
	if err != nil {
		return nil, err
	}
	card[0].Images = imgs
	targets, err := m.GetCardTargetByCardID(cardid)
	if err != nil {
		return nil, err
	}
	card[0].Targets = targets
	share, err := m.GetCardShareByCardID(cardid)
	if err != nil {
		return nil, err
	}
	card[0].Share = share
	return &card[0], err
}

// AddCard 新增卡信息
func (m *CardDAO) AddCard(card *model.TaCouponCard) error {
	if card.CardId == "" {
		card.CardId = guid.NewGUIDString()
	}

	userRaw := m.ctx.Get("user")
	if userRaw != nil {
		user := userRaw.(model.SysUser)
		card.CreateUser = user.UserId
	}

	card.Status = models.STATUS_NORMAL
	card.CreateDate = time.Now().Local()

	if _, err := m.db.Insert(card); err != nil {
		return err
	}

	return nil
}

// UpdateCard 更新优惠券
func (m *CardDAO) UpdateCard(card *model.TaCouponCard, cols []string) error {
	if card.CardId == "" {
		return errors.New("无卡信息")
	}

	if _, err := m.db.Cols(cols...).Where("card_id=?", card.CardId).Update(card); err != nil {
		return err
	}

	return nil
}

// DeleteCard 删除卡
func (m *CardDAO) DeleteCard(id string) error {
	if id == "" {
		return errors.New("无卡信息")
	}

	card := model.TaCouponCard{
		CardId: id,
		Status: models.STATUS_DEL,
	}

	if err := m.UpdateCard(&card, []string{"status"}); err != nil {
		return err
	}

	return nil
}

// SaveCustomerCard 保存用户卡
func (m *CardDAO) SaveCustomerCard(card *model.TaCustomerCard) error {
	if card.CustomerCardId == "" {
		card.CustomerCardId = guid.NewGUIDString()
	}

	card.Status = models.STATUS_NORMAL

	if _, err := m.db.Insert(card); err != nil {
		return err
	}

	return nil
}

// SaveCustomerCards 保存用户卡
func (m *CardDAO) SaveCustomerCards(cards []model.TaCustomerCard) error {
	for i := range cards {
		if cards[i].CustomerCardId == "" {
			cards[i].CustomerCardId = guid.NewGUIDString()
		}

		cards[i].Status = models.STATUS_NORMAL
	}

	if _, err := m.db.Insert(cards); err != nil {
		return err
	}

	return nil
}

// CheckCustCard 检查客户是否已经有了该卡券
// 如果用户有了卡券, 但是用过了，或者卡券过期了，仍然可以再次拥有
func (m *CardDAO) CheckCustCard(custID, CardID string, startDate, endDate time.Time) ([]model.TaCustomerCard, error) {
	var cps []model.TaCustomerCard
	query := `
		SELECT
			*
		FROM
			ta_customer_card t
		WHERE
			t.card_id = ?
		AND t.customer_id = ?
		AND t.status = ?
		AND (
			(t.start_date BETWEEN ? AND ?)
			OR (t.end_date BETWEEN ? AND ?)
			OR (
				t.start_date <= ?
				AND t.end_date >= ?
			)
		)
	`

	if err := m.db.SQL(
		query,
		CardID,
		models.STATUS_NORMAL,
		custID,
		startDate,
		endDate,
		startDate,
		endDate,
		startDate,
		endDate,
	).Find(&cps); err != nil {
		return nil, err
	}

	return cps, nil
}

// 案场可用卡信息
type CaseUsableCard struct {
	model.TaCouponCard `xorm:"extends"`
	UserSendCount      int
	UserUsedCount      int
}

// GetCaseUsableCard 获取案场可用卡信息
func (m *CardDAO) GetCaseUsableCard(caseid string) ([]CaseUsableCard, error) {
	var cards []CaseUsableCard
	sql := `select * from ta_coupon_card where case_id=? and status=? and end_date>now() order by create_date limit `
	err := m.db.Sql(sql).Find(&cards)
	// err := m.db.Where("case_id=?", caseid).And("status=?", models.STATUS_NORMAL).And("end_date>now()").Desc("create_date").Find(&cards)
	return cards, err
}
