package coupon

import (
	"errors"
	"fmt"
	"spaceofcheng/services/models"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/utils"
	"strconv"
	"strings"
	"time"

	"github.com/yl10/kit/guid"

	"github.com/go-xorm/xorm"
)

// CouponDAO 当前数据库操作对象
type CouponDAO struct {
	ctx *utils.Context
	db  *xorm.Session
}

// NewCouponDAO New Inst
func NewCouponDAO(ctx *utils.Context) *CouponDAO {
	return &CouponDAO{
		ctx: ctx,
		db:  ctx.DB,
	}
}

// GetCouponByID 获取卡券信息
func (m *CouponDAO) GetCouponByID(id string) (*model.TaCoupon, error) {
	cp := model.TaCoupon{}

	if _, err := m.db.Where("coupon_id=?", id).Get(&cp); err != nil {
		return nil, err
	}

	return &cp, nil
}

// GetCouponInfoByID 获取优惠券详情
func (m *CouponDAO) GetCouponInfoByID(id string) (*CouponInfo, error) {
	cp := new(CouponInfo)

	query := `
		SELECT
			*
		FROM
			ta_coupon
		WHERE
			coupon_id = ?
	`

	if _, err := m.db.SQL(query, id).Get(cp); err != nil {
		return nil, err
	}

	// 规则
	var rules []model.TaCouponRule
	if err := m.db.Where("coupon_id=?", id).And("status=?", models.STATUS_NORMAL).Find(&rules); err != nil {
		return nil, err
	}

	// 图片
	var imgs []model.TaCouponImage
	if err := m.db.Where("coupon_id=?", id).Find(&imgs); err != nil {
		return nil, err
	}

	// 关联目标
	var targes []model.TaCouponTarget
	if err := m.db.Where("coupon_id=?", id).Find(&targes); err != nil {
		return nil, err
	}

	// 分享
	share := new(model.TaCouponShare)
	if _, err := m.db.Where("coupon_id=?", id).And("status=?", models.STATUS_NORMAL).Get(share); err != nil {
		return nil, err
	}

	cp.Targets = targes
	cp.Images = imgs
	cp.Rules = rules
	cp.Share = *share

	return cp, nil
}

// GetCouponList 获取优惠券列表
func (m *CouponDAO) GetCouponList(filters []string, limit []int) ([]model.TaCoupon, int64, error) {
	var cps []model.TaCoupon

	filterString := ""
	if len(filters) > 0 {
		filterString = strings.Join(filters, " and ")
		filterString += " and "
	}

	sql := `select * from ta_coupon where %s status > ` + strconv.Itoa(models.STATUS_DEL) + ` order by create_date desc`

	total, err := utils.NewPageNaviEngine(m.ctx).GetPageList(&cps, fmt.Sprintf(sql, filterString), limit)
	if err != nil {
		return nil, 0, err
	}

	return cps, total, nil
}

// SaveCoupon 保存优惠券
func (m *CouponDAO) SaveCoupon(cp *model.TaCoupon) error {
	if cp.CouponId == "" {
		cp.CouponId = guid.NewGUIDString()
	}

	userRaw := m.ctx.Get("user")
	if userRaw != nil {
		user := userRaw.(model.SysUser)
		cp.CreateUser = user.UserId
	}

	org := m.ctx.Get("org").(model.SysOrg)
	cp.OrgId = org.OrgId

	cp.Status = models.STATUS_NORMAL
	cp.CreateDate = time.Now().Local()

	if _, err := m.db.Insert(cp); err != nil {
		return err
	}

	return nil
}

// UpdateCoupon 更新优惠券
func (m *CouponDAO) UpdateCoupon(cp *model.TaCoupon, cols []string) error {
	if cp.CouponId == "" {
		return errors.New("无优惠券信息")
	}

	if _, err := m.db.Cols(cols...).Where("coupon_id=?", cp.CouponId).Update(cp); err != nil {
		return err
	}

	return nil
}

// DeleteCoupon 删除优惠券
func (m *CouponDAO) DeleteCoupon(id string) error {
	if id == "" {
		return errors.New("无优惠券信息")
	}

	cp := model.TaCoupon{
		CouponId: id,
		Status:   models.STATUS_DEL,
	}

	if err := m.UpdateCoupon(&cp, []string{"status"}); err != nil {
		return err
	}

	return nil
}

// SaveCustomerCoupon 保存用户卡券
func (m *CouponDAO) SaveCustomerCoupon(cp *model.TaCustomerCoupon) error {
	if cp.CustomerCouponId == "" {
		cp.CustomerCouponId = guid.NewGUIDString()
	}

	cp.Status = models.STATUS_NORMAL

	if _, err := m.db.Insert(cp); err != nil {
		return err
	}

	return nil
}

// SaveCustomerCoupons 保存用户卡券
func (m *CouponDAO) SaveCustomerCoupons(cps []model.TaCustomerCoupon) error {
	for i := range cps {
		if cps[i].CustomerCouponId == "" {
			cps[i].CustomerCouponId = guid.NewGUIDString()
		}

		cps[i].Status = models.STATUS_NORMAL
	}

	if _, err := m.db.Insert(cps); err != nil {
		return err
	}

	return nil
}

// CheckCustCoupon 检查客户是否已经有了该卡券
// 如果用户有了卡券, 但是用过了，或者卡券过期了，仍然可以再次拥有
func (m *CouponDAO) CheckCustCoupon(custID, couponID string, startDate, endDate time.Time) ([]model.TaCustomerCoupon, error) {
	var cps []model.TaCustomerCoupon
	query := `
		SELECT
			*
		FROM
			ta_customer_coupon t
		WHERE
			t.coupon_id = ?
		AND t.customer_id = ?
		AND t.status = ?
		AND (
			(t.start_date BETWEEN ? AND ?)
			OR (t.end_date BETWEEN ? AND ?)
			OR (
				t.start_date <= ?
				AND t.end_date >= ?
			)
		)
	`
	if err := m.db.SQL(
		query,
		couponID,
		custID,
		models.STATUS_NORMAL,
		startDate,
		endDate,
		startDate,
		endDate,
		startDate,
		endDate,
	).Find(&cps); err != nil {
		return nil, err
	}

	return cps, nil
}

// SaveCouponRecoreds 保存优惠券记录
func (m *CouponDAO) SaveCouponRecoreds(recs []model.TaCouponGiveRecord) error {
	for i := range recs {
		if recs[i].RecId == "" {
			recs[i].RecId = guid.NewGUIDString()
		}
	}

	if _, err := m.db.Insert(&recs); err != nil {
		return err
	}

	return nil
}
